//
// 2011/6/26 Ryuki Tanaka
//
// まず、Singletonを呼び出す。
// その中のRootManagerをgetする。
// あとは、それっぽいところでRootManagerの中身を順番に呼んでいくだけ。
// データの格納はrootmanager->Imap.Input("px", px);とか
// (Input()はDataMapの関数だが気にしない。第2に引数は値渡し。)
// Dmap,Imapはデータの一時保管庫で、それぞれdoubleとintに対応。
// 他の型はRootManager.hh参照。
// Geant用の型(G4threevector)とかを使うなら継承して使うのが良いかと。
// (RootManagerForGeant4.hh参照)
// 細かいところはsample sourceをみて。
//
// 注意）inputするときにどのmapでもどの型も受け付けてしまう。
// そのため、明示的に型を変換しないといけない。
// 例えば、doubleのものをintとしてImapに入れるなら
// Imap.input("x", (int)x);
// とか。
//
#ifndef ROOTMANAGER_HH
#define ROOTMANAGER_HH

#include "TFile.h"
#include "TTree.h"

#include <string>
#include <map>
#include <sstream>

#include "DataMap.hh"

/************************************************************
  - B : an 8 bit signed integer (Char_t)
  - b : an 8 bit unsigned integer (UChar_t)
  - S : a 16 bit signed integer (Short_t)
  - s : a 16 bit unsigned integer (UShort_t)
  - I : a 32 bit signed integer (Int_t)
  - i : a 32 bit unsigned integer (UInt_t)
  - F : a 32 bit floating point (Float_t)
  - D : a 64 bit floating point (Double_t)
  - L : a 64 bit signed integer (Long64_t)
  - l : a 64 bit unsigned integer (ULong64_t)
  - O : a boolean (Bool_t)
************************************************************/
typedef DataMap<std::string, std::pair<Char_t   , std::string> > Bmap;
typedef DataMap<std::string, std::pair<UChar_t  , std::string> > bmap;
typedef DataMap<std::string, std::pair<Short_t  , std::string> > Smap;  
typedef DataMap<std::string, std::pair<UShort_t , std::string> > smap;  
typedef DataMap<std::string, std::pair<Int_t    , std::string> > Imap;
typedef DataMap<std::string, std::pair<UInt_t   , std::string> > imap;
typedef DataMap<std::string, std::pair<Float_t  , std::string> > Fmap;
typedef DataMap<std::string, std::pair<Double_t , std::string> > Dmap;
typedef DataMap<std::string, std::pair<Long64_t , std::string> > Lmap;
typedef DataMap<std::string, std::pair<ULong64_t, std::string> > lmap;
typedef DataMap<std::string, std::pair<Bool_t   , std::string> > Omap;

//RootManager class
class RootManager
{
public:
  void Initialize();
  void CreateTree(std::string treename, std::string title);
  template <class Name, class Data, class DataName>
  void Input(DataMap<Name, std::pair<Data, DataName> > *datamap,
	     std::string name, Data data, std::string dataname);
  template <class Name, class Data, class DataName>
  void CreateBranch(std::string treename, DataMap<Name, std::pair<Data, DataName> > *datamap);
  void Fill();
  void Fill(std::string treename);
  void CreateFile(std::string filename, std::string option);
  void Write();
  void Write(std::string treename);

public:
  RootManager();
  ~RootManager();

private:
  DataMap<std::string, TTree*> Treemap;
  DataMap<std::string, TFile*> Filemap;

private:
  TTree* GetTree(std::string treename);
  TFile* GetFile(std::string filename);
  void CloseTree(std::string treename);
  void CloseFile(std::string filename);
  void DeleteTree();
  void DeleteFile();
};

template <class Name, class Data, class DataName>
inline void RootManager::Input(DataMap<Name, std::pair<Data, DataName> > *datamap,
	   std::string name, Data data, std::string dataname){
  std::pair<Data, std::string> pair(data, dataname);
  datamap->Input(name, pair);
}

template <class Name, class Data, class DataName>
inline void RootManager::CreateBranch
(std::string treename, DataMap<Name, std::pair<Data, DataName> > *datamap)
{
  TTree* t = GetTree(treename);
  typename DataMap<Name, std::pair<Data, DataName> >::iterator it;
  it = datamap->begin();
  while(it != datamap->end()){
    std::ostringstream os;
    os << (it->second).second;
    t->Branch(it->first.c_str(), &((it->second).first), (os.str()).c_str());
    ++it;
  }
}

#endif
