//
// 2011/6/26 Ryuki Tanaka
//
#include "RootManager.hh"

RootManager::RootManager() : Treemap(), Filemap()
{;}

RootManager::~RootManager(){Initialize();}

void RootManager::Initialize()
{
  DeleteTree();
  DeleteFile();
}  

void RootManager::CreateTree(std::string treename, std::string title)
{
  TTree* t = new TTree(treename.c_str(), title.c_str());
  Treemap.Input(treename, t);
}

void RootManager::Fill()
{
  DataMap<std::string, TTree*>::iterator it = Treemap.begin();
  while(it != Treemap.end()){
    (it->second)->Fill();
    ++it;
  }
}

void RootManager::Fill(std::string treename)
{
  TTree* t = GetTree(treename);
  t->Fill();
}

void RootManager::CreateFile(std::string filename, std::string option)
{
  TFile* f = new TFile(filename.c_str(), option.c_str());
  Filemap.Input(filename, f); 
}

void RootManager::Write()
{
  DataMap<std::string, TTree*>::iterator it = Treemap.begin();
  while(it != Treemap.end()){
    this->Write(it->first);
    ++it;
  }
}

void RootManager::Write(std::string treename)
{
  TTree* t = GetTree(treename);
  t->Write();
  CloseTree(treename);
}

TTree* RootManager::GetTree(std::string treename)
{
  return Treemap.Output(treename);
}

TFile* RootManager::GetFile(std::string filename)
{
  return Filemap.Output(filename);
}

void RootManager::CloseTree(std::string treename)
{
  typename DataMap<std::string, TTree*>::iterator it = Treemap.Find(treename);
  delete it->second;
  it->second = 0;
  Treemap.erase(it);
}

void RootManager::CloseFile(std::string filename)
{
  typename DataMap<std::string, TFile*>::iterator it = Filemap.Find(filename);
  delete it->second;
  it->second = 0;
  Filemap.erase(it);
}

void RootManager::DeleteTree()
{
  DataMap<std::string, TTree*>::iterator it = Treemap.begin();
  if(it != Treemap.end()){
    CloseTree(it->first);
    ++it;
  }
}

void RootManager::DeleteFile()
{
  DataMap<std::string, TFile*>::iterator it = Filemap.begin();
  if(it != Filemap.end()){
    CloseFile(it->first);
    ++it;
  }
}
